/*
 * Created on May 9, 2004
 * ComplexToPoly.java
 *//* 
* This  program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published  by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be  useful,  but
* WITHOUT   ANY   WARRANTY;   without  even  the  implied  warranty  of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.   See  the  GNU
* General Public License for more details.
*
* You  should  have  received  a copy of the GNU General Public License
* along  with  this  program;  if  not,  write  to  the  Free  Software
* Foundation,  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA.
*/
package pzapplet;

import ORG.netlib.math.complex.*;

/**
 * This class calculates a single polynomial from any number of real
 * or complex conjugate poles.  
 * 
 * Complex poles must have a conjugate, as 
 * this class uses that assumption to make fewer calculations.
 * 
 * 
 * This takes a complex array of values and makes available a
 *  single polynomial.
 * 
 * The terms of the polynomial are represented in an array, where
 * the index of the array corresponds to the value of that power.
 * 
 * Representation: poly[x]=y is the equivalent of y*(s^x).
 * 
 * Additional data stored is the degree of the polynomial, to avoid 
 * needing to search the polynomial for the highest non-zero 
 * coefficient.
 * 
 * The number of points at the origin are also available through
 * a separate method.
 * @author Brian Williams
 *
 */
public class ComplexToPoly {

	private double[] poly = new double[1000];
	private int degree = 0;
	private int zeros = 0; // Number of (0.0, 0.0) terms
	private boolean doneSomething = false;

	/**
	 * Creates a new ComplexToPoly with the given complex values
	 * 
	 * @param complex Complex[] of values to turn into a polynomial
	 */
	public ComplexToPoly(Complex[] complex) {

		poly[0] = 1.0;

		for (int i = 0; i < complex.length; i++) {
			Complex arg = complex[i];
			if (arg.im() < 0.0) { // If below x-axis
				// Do nothing, will multiply it and conjugate later
			} else if (arg.re()==0.0 && arg.im()==0.0) {
				this.zeros++;
				doneSomething=true;
				
			} else { // If on x-axis or above
				multiply(arg);
				doneSomething=true;
			}
		}
		if(!doneSomething) {
			//poly[0]=0.0;
		}
		
		calculateZeros(this.zeros);
	}

	private void multiply(Complex arg) {
		double[] tempPoly = new double[1000];
		tempPoly[0]=0.0;
		double a = -arg.re();
		double b = -arg.im();
		double aSquaredPlusBSquared = a * a + b * b;
		if (b == 0.0 && a != 0.0) {
			// s*(1/a) + 1
			//System.out.println("On the real axis...");
			for (int i = 0; i <= degree; i++) {
				tempPoly[i + 1] = (tempPoly[i+1]) + (poly[i] * (1.0/a));
				tempPoly[i]=tempPoly[i] + 1.0*poly[i];
			}
			poly=tempPoly;
			degree += 1;
		} else if (b != 0.0) {
			// s^2*(1/(a^2+b^2)) + s*(2a/(a^2+b^2)) + 1
			//System.out.println("Not on the real axis...");
			for (int j = 0; j <= degree; j++) {
				tempPoly[j + 2] =
					(tempPoly[j + 2]) + ((1.0 / aSquaredPlusBSquared) * poly[j]);
				tempPoly[j + 1] =
					(tempPoly[j+1])
						+ ((2.0 * a / aSquaredPlusBSquared) * poly[j]);
				tempPoly[j] = tempPoly[j] + 1.0*poly[j];
			}
			poly=tempPoly;
			degree += 2;
		} else {
			//System.out.println("Must be on the origin...");
		}
	}
	
	// Shifts polynomial by how many ever zero terms exist
	private void calculateZeros(int num) {
		for (int i = 0; i<num; i++) {
			degree++;
			for (int j=998; j>=0; j--) {
				poly[j+1]=poly[j];
			}
			poly[0]=0.0;
		}
	}

	/**
	 * @return the highest degree of the polynomial
	 */
	public int getDegree() {
		return degree;
	}

	/**
	 * @return coefficients of the polynomial from lowest to highest
	 */
	public double[] getPoly() {
		double[] trimmedPoly = new double[degree + 1];
		for (int i = 0; i <= degree; i++) {
			trimmedPoly[i] = poly[i]/poly[degree];
		}
		return trimmedPoly;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String string = "";
		for (int i = degree; i >= 0; i--) {
			if (poly[i] != 0.0) {
				if (string != "") {
					if (poly[i] > 0.0) {
						string = string + "+";
					}
				}
				string = string + poly[i] + "s^" + i;
			}
		}
		
		return string;
	}

	/**
	 * @return this in readable form, scaled so that the highest-order
	 * 		polynomial is 1.
	 */
	public String toNormalizedString() {
		String string = "";
		for (int i = degree; i >= 0; i--) {
			if (poly[i] != 0.0) {
				if (string != "") {
					if (poly[i]/poly[degree] > 0.0) {
						string = string + "+";
					}
				}
				string = string + poly[i]/poly[degree] + "s^" + i;
			}
		}

		return string;
	}

	/**
	 * @return int representing the number of points at the origin
	 */
	public int getZeros() {
		return zeros;
	}

}
