/*
 * Created on Feb 13, 2004
 * FreqResponse.java
 *//* 
* This  program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published  by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be  useful,  but
* WITHOUT   ANY   WARRANTY;   without  even  the  implied  warranty  of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.   See  the  GNU
* General Public License for more details.
*
* You  should  have  received  a copy of the GNU General Public License
* along  with  this  program;  if  not,  write  to  the  Free  Software
* Foundation,  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA.
*/
package pzapplet;

import ORG.netlib.math.complex.*;

/**
 * Represents a frequency response of a system.  
 * 
 * The data is kept as arrays of data points.  
 * Once created, the system cannot be altered.
 * 
 * @author Brian Williams
 *
 */
public class FreqResponse {

	// Storage for related information
	// Frequency Vector holds omega values
	private double[] freqs; // Holds w for each mag/phase pair
	private double[] mags; // Holds mag H(w) for each freq value
	private double[] phases; // Holds phase H(w) for each freq value

	private Complex[] poles; // Set of poles used in H(w)
	private Complex[] zeros; // Set of zeros used in H(w)

	private int low;
	private int high;
	private int res;
	
	private double resFreq;

	/**
	 * This creates a FreqResponse of discrete data points over a range of
	 * frequencies, given the poles and zeros passed to this.
	 * 
	 * <br>
	 * @param zeros - Array of Complex objects representing zeros of a system
	 * @param poles - Array of Complex objects representing poles of a system
	 * 
	 * 
	 */
	public FreqResponse(Complex[] zeros, Complex[] poles) {
		this.zeros = zeros;
		this.poles = poles;
		this.low = -2;
		this.high = 2;
		this.res = 100;
		this.freqs = this.CreateFreqs(this.low, this.high, this.res);
		this.mags = this.CreateMags(this.freqs);
		this.phases = this.CreatePhases(this.freqs);
	}
	
	/**
	 * This creates a FreqResponse with the specified magnitudes and phases
	 * at each frequency ranging from 10^(low) to 10^(high).
	 * 
	 * <br>MUST FOLLOW -- Length of mags == length of phases
	 *      -- Lowest and highest frequency points must be powers of 10
	 *      -- Length of mags must be evenly divisible by (high - low)
	 * 
	 * @param mags - Array of doubles representing magnitude values
	 * @param phases - Array of doubles representing phase values
	 * @param low - int representing the lowest frequency (power of 10)
	 * @param high - int representing the highest frequency (power of 10)
	 *
	 */
	public FreqResponse(double[] mags, double[] phases, int low, int high) {
		this.mags=mags;
		this.phases=phases;
		this.low=low;
		this.high=high;
		this.res=(mags.length-1)/(high-low);
		if(mags.length==phases.length) {
			this.freqs=this.CreateFreqs(this.low, this.high, this.res);
		} else {
			System.out.println("Warning: different mag/phase lengths");
		}
	}

	// Creates the list of frequencies given the lowest frequency and
	// highest frequency, as powers of 10.  The res is the resolution,
	// or number of data points each decade.
	private double[] CreateFreqs(int low, int high, int res) {
		double[] freq = new double[(high-low)*res+1];
		for (int w = 0; w <= (high-low)*res; w++) {
			double doub = Math.pow(10,low) * Math.pow(10, w * 1/(double)res);
			freq[w] = doub;
			//System.out.println(doub+ "  for w= " + w);
		}
		return freq;
	}

	// Creates the list of magnitudes corresponding to each frequency value
	private double[] CreateMags(double[] freqList) {
		double[] mag = new double[freqList.length];
		for (int i = 0; i < freqList.length; i++) {
			//  Take Poles, Zeros and Freqs 
			//  Calculate Magnitude at each freq
			//  20 log mag(H(jw)) = 
			//     20 log mag(s/zn + 1) - 20 log mag(s/pn + 1)
			// Special cases for poles/zeros at the origin
			
			double freq = freqList[i];
			double magnitude = 0.0;
			Complex origin = new Complex(0.0, 0.0);
			Complex jw = new Complex(0, freq);
			Complex one = new Complex(1.0, 0.0);
			for (int k = 0; k < poles.length; k++) {
				Complex pole = poles[k];
				if (pole.equals(origin, 0.01)) {
					magnitude += 20.0
						* Math.log((one.div(jw)).abs())
						/ Math.log(10.0);
				} else {
					Complex term = jw.div(pole).add(one);
					magnitude -= 20 * Math.log(term.abs()) / Math.log(10.0);
				}

			}
			for (int l = 0; l < zeros.length; l++) {
				Complex zero = zeros[l];
				if (zero.equals(origin, 0.01)) {
					magnitude -= 20.0
						* Math.log((one.div(jw)).abs())
						/ Math.log(10.0);
				} else {
					Complex term = jw.div(zero).add(one);
					magnitude += 20.0 * Math.log(term.abs()) / Math.log(10.0);
				}
			}
			mag[i] = magnitude;
		}
		return mag;
	}
	
	// Creates the list of phases corresponding to each frequency value
	private double[] CreatePhases(double[] freqList) {
		double[] phases = new double[freqList.length];
		for (int j = 0; j < freqList.length; j++) {
			// Calculate phase from each pole/zero
			// Sum together to get total phase at each freq
			// angle(jw) = arctan(imag(zn)/real(zn))
			double freq = freqList[j];
			Complex origin = new Complex(0.0, 0.0);
			Complex jw = new Complex(0, freq);

			double phase = 0.0;
			
			for (int k = 0; k < poles.length; k++) {
				Complex pole = poles[k].neg();
				if (pole.equals(origin, 0.01)) {
					//System.out.println("ORIGIN!");
					phase -= Math.PI/2.0;
				} else {
					Complex comp = (pole.add(jw));
					phase -= Math.atan(comp.im() / comp.re());
				}
			}
			for (int l = 0; l < zeros.length; l++) {
				Complex zero = zeros[l].neg();
				if (zero.equals(origin, 0.01)) {
					phase += Math.PI/2.0;
				} else {
					Complex comp = zero.add(jw);
					phase += Math.atan(comp.im() / comp.re());
				}
			}
			phase = Math.toDegrees(phase);
			phases[j] = phase;
		}
		return phases;
	}
	/**
	 * Returns a double[] of each frequency value iteration
	 */
	public double[] getFreqs() {
		return freqs;
	}

	/**
	 * @return a double[] of each magnitude value iteration
	 */
	public double[] getMags() {
		return mags;
	}

	/**
	 * @return a double[] of each phase value iteration
	 */
	public double[] getPhases() {
		return phases;
	}

	/**
	 * @return Mathematical log of highest frequency used in calculation
	 */
	public int getHigh() {
		return high;
	}

	/**
	 * @return Mathematical log of lowest frequency used in calculation
	 */
	public int getLow() {
		return low;
	}

	/**
	 * @return Resolution of omega used to calculate frequency response
	 */
	public int getRes() {
		return res;
	}

}
