/*
 * Created on Feb 15, 2004
 * PZSPlaneGraph.java
 *//* 
* This  program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published  by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be  useful,  but
* WITHOUT   ANY   WARRANTY;   without  even  the  implied  warranty  of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.   See  the  GNU
* General Public License for more details.
*
* You  should  have  received  a copy of the GNU General Public License
* along  with  this  program;  if  not,  write  to  the  Free  Software
* Foundation,  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA.
*/
package pzapplet;

import javax.swing.JPanel;
import java.awt.*;
import java.awt.event.*;
import javax.swing.BorderFactory;
import java.util.Vector;

/**
 * Contains the S-Plane graph and allows clicking to
 * modify the graph.
 * 
 * @author Brian Williams
 *
 */
public class PZSPlaneGraph
	extends JPanel
	implements MouseListener, MouseMotionListener {

	// Variables to hold pole or zero being dragged
	private boolean activeZero = false;
	private boolean activePole = false;
	private Coord actCoord;

	// Reference to PZApplet
	private PoleZeroApplet pzApplet;

	// Settings defining pixel range for deleting and snapping to axes
	private final int AXES_SNAP_DISTANCE = 5;
	private final int CLICK_RANGE = 5;

	// Size of the panel
	private int height;
	private int width;

	/**
	 * @param pzApp - Reference to PZApplet, cannot be null
	 * 
	 * Creates a new PZSPlaneGraph for inputting the poles and zeros of
	 * the system.  The input method is clicking on the graph and the 
	 * result of the click is determined by the mode of the applet.
	 *
	 */
	public PZSPlaneGraph(PoleZeroApplet pzApp) {
		pzApplet = pzApp;
		addMouseListener(this);
		addMouseMotionListener(this);
		height = 200;
		width = 201;
		this.setBorder(BorderFactory.createLineBorder(Color.BLACK));
	}

	/* 
	 * Draws the s-plane and poles and zeros of the system
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D)g.create();
		// Get Poles and Zeros to graph
		Vector poles = pzApplet.getPoles();
		Vector zeros = pzApplet.getZeros();

		// Fill in Background
		g2.setColor(Color.WHITE);
		g2.fillRect(0, 0, width, height);

		// Draw axes and labels
		g2.setColor(Color.BLACK);
		g2.drawLine(width / 2, 0, width / 2, height);
		g2.drawLine(0, height / 2, width, height / 2);
		g2.setFont(new Font(null, 00, 10));
		g2.drawString("10", 102, 12);
		g2.drawString("-10", 103, 198);
		g2.drawString("-10", 2, 112);
		g2.drawString("10", 184, 112);

		// Drawing Poles on Graph
		if (poles.size() >= 1) {
			g2.setColor(Color.BLUE);
			for (int i = 0; i < poles.size(); i++) {
				Coord a = (Coord)poles.get(i);
				int aX = new Double(a.getX()).intValue();
				int aY = new Double(a.getY()).intValue();
				g2.drawLine(aX - 5, aY + 5, aX + 5, aY - 5);
				g2.drawLine(aX + 5, aY + 5, aX - 5, aY - 5);
			}
			g2.setColor(Color.BLACK);
		}

		// Drawing Zeros on Graph
		if (zeros.size() >= 1) {
			g2.setColor(Color.RED);
			for (int i = 0; i < zeros.size(); i++) {
				Coord a = (Coord)zeros.get(i);
				int aX = new Double(a.getX()).intValue();
				int aY = new Double(a.getY()).intValue();
				g2.drawOval(aX - 5, aY - 5, 10, 10);
			}
			g2.setColor(Color.BLACK);
		}
		g2.dispose();
	}

	// Sets dimentions of graph to be a fixed size
	public Dimension getPreferredSize() {
		return new Dimension(width, height);
	}
	public Dimension getMinimumSize() {
		return new Dimension(width, height);
	}
	public Dimension getMaximumSize() {
		return new Dimension(width, height);
	}

	/* 
	 * If mouse clicked, checks the current mode, and either adds or deletes
	 * a pole/zero.  Dragging is not involved in this method.
	 */
	public void mouseClicked(MouseEvent arg0) {
		String mode = pzApplet.getMode();
		int x = arg0.getX();
		int y = arg0.getY();

		// Sets click to a click on the axis if within distance
		Coord click = new Coord(x, y);
		Coord clickOnAxis = new Coord(x, height / 2);
		if (click.withinRangeOf(clickOnAxis, AXES_SNAP_DISTANCE)) {
			click = clickOnAxis;
		}
		
		// Adds pole
		if (mode == "Poles") {
			pzApplet.addPolePair(click, height);
		}
		
		// Adds zero
		if (mode == "Zeros") {
			pzApplet.addZeroPair(click, height);
		}

		//	Deleting a pole/zero
		if (mode == "Delete") {
			boolean deleted = false;
			for (int i = 0; i < pzApplet.getZeros().size(); i++) {
				Coord zero = (Coord)pzApplet.getZeros().get(i);
				if (click.withinRangeOf(zero, CLICK_RANGE)) {
					pzApplet.deleteZeroPair(zero, height, true);
					deleted = true;
					break;
				}
			}
			// If didn't delete a zero, try to delete a pole
			if (!deleted) {
				for (int i = 0; i < pzApplet.getPoles().size(); i++) {
					Coord pole = (Coord)pzApplet.getPoles().get(i);
					if (click.withinRangeOf(pole, CLICK_RANGE)) {
						pzApplet.deletePolePair(pole, height, true);
						break;
					}
				}
			}
		}
	}

	/* 
	 */
	public void mouseEntered(MouseEvent arg0) {
		// Nothing to do here
	}

	/* 
	 * If dragging, "drops" the pole or zero if one was being dragged
	 */
	public void mouseExited(MouseEvent arg0) {
		activePole = false;
		activeZero = false;
		actCoord = null;
	}

	/* 
	 * When pressing mouse button, if in dragging mode, "locks on" to
	 * the nearest pole/zero to the click
	 */
	public void mousePressed(MouseEvent arg0) {
		String mode = pzApplet.getMode();
		// If in the drag and drop mode
		if (mode == "Dragging") {
			int x = arg0.getX();
			int y = arg0.getY();
			Coord click = new Coord(x, y);

			// Find a pole to lock on to
			for (int i = 0; i < pzApplet.getPoles().size(); i++) {
				Coord pole = (Coord)pzApplet.getPoles().get(i);
				if (click.withinRangeOf(pole, CLICK_RANGE)) {
					actCoord = pole;
					activePole = true;
					break;
				}
			}

			// If no pole around, find a zero to lock on to
			if (!activePole) {
				for (int i = 0; i < pzApplet.getZeros().size(); i++) {
					Coord zero = (Coord)pzApplet.getZeros().get(i);
					if (click.withinRangeOf(zero, CLICK_RANGE)) {
						actCoord = zero;
						activeZero = true;
						break;
					}
				}
			}
		} // end of if("dragging"==mode)
	}

	/* 
	 * Resets the dragging variables when mouse is released
	 */
	public void mouseReleased(MouseEvent arg0) {
		activePole = false;
		activeZero = false;
		actCoord = null;
	}

	/* 
	 * If in dragging mode, this method moves around the pole or zero
	 */
	public void mouseDragged(MouseEvent arg0) {
		String mode = pzApplet.getMode();
		if (mode == "Dragging" && (activePole || activeZero)) {
			int x = arg0.getX();
			int y = arg0.getY();
			Coord click = new Coord(x, y);
			Coord clickOnAxis = new Coord(x, height / 2);

			// If click is close to axis, "move" click to axis
			if (click.withinRangeOf(clickOnAxis, AXES_SNAP_DISTANCE)) {
				click = clickOnAxis;
			}

			// Moves the pole, if dragging a pole
			if (activePole) {
				pzApplet.deletePolePair(actCoord, height, false);
				pzApplet.addPolePair(click, height);
				actCoord = click;
				//repaint();
			}
			// Moves a zero, if moving a zero
			else if (activeZero) {
				pzApplet.deleteZeroPair(actCoord, height, false);
				pzApplet.addZeroPair(click, height);
				actCoord = click;
				//repaint();
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent arg0) {
		// Nothing to do here
	}

	/**
	 * Updates the graph and redraws poles and zeros.
	 */
	public void updateGraphics() {
		this.repaint();
	}

}
