/*
 * Created on Feb 13, 2004
 * StepResponse.java
 *//* 
* This  program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published  by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be  useful,  but
* WITHOUT   ANY   WARRANTY;   without  even  the  implied  warranty  of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.   See  the  GNU
* General Public License for more details.
*
* You  should  have  received  a copy of the GNU General Public License
* along  with  this  program;  if  not,  write  to  the  Free  Software
* Foundation,  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA.
*/
package pzapplet;

import ORG.netlib.math.complex.*;
import Jama.*;

/**
 * Calculates and stores the step response of a given system of poles 
 * and zeros.
 * 
 * @author Brian Williams
 *
 */
public class StepResponse {

	private Complex[] poles; // Set of poles used in H(w)
	private Complex[] zeros; // Set of zeros used in H(w)
	private boolean errorMsg=false;
	private double[] stepResponseData;  // Stores step response
	private double gain=1.0;
	
	/**
	 * This creates the step response data for a given system.
	 * 
	 * The system must be represented by two arrays of complex
	 * numbers, the poles and zeros of the system.
	 * 
	 * 
	 * @param zeros - Complex[] representing the zeros of the system
	 * @param poles - Complex[] representing the poles of the system
	 */
	public StepResponse(Complex[] zeros, Complex[] poles) {
		this.poles=poles;
		this.zeros=zeros;
	
		// First, create polynomials out of poles and zeros
		ComplexToPoly polesPoly = new ComplexToPoly(poles);
		ComplexToPoly zerosPoly = new ComplexToPoly(zeros);
		if (zerosPoly.getDegree()>polesPoly.getDegree()) {
			errorMsg=true;
		} else if(polesPoly.getDegree()==0){
			stepResponseData=new double[401];	
		} else {
			RungeKutta(zerosPoly,polesPoly);
		}
	}
	
	private void RungeKutta(ComplexToPoly zeros, ComplexToPoly poles) {
		double step = 0.01; // step size
		int N = poles.getDegree(); // Degree of denominator poly
		
		//Matrix poleMatrix = new Matrix(poles.getPoly(),1);
		Matrix zeroMatrix = new Matrix(N+1,1);
		
		// Fill zero Matrix with zeros, if zero poly is lower degree
		// And adjust for the gain of the system
		for (int i = 0; i<zeros.getPoly().length; i++) {
			this.gain = zeros.getPoly()[zeros.getZeros()];
			zeroMatrix.set(N-i,0,zeros.getPoly()[i]*poles.getPoly()[poles.getZeros()]/zeros.getPoly()[zeros.getZeros()]);
		}
		
		// Matrices in X-Dot= A*x+ b
		Matrix A = new Matrix(N,N);
		Matrix b = new Matrix(N,1);
		
		Matrix F = new Matrix(N+1,N+1);
		
		// Set Matrix A
		// First, set the identity matrix in upper-right corner
		// (an N-1 by N-1 identity matrix)
		for(int i=1 ; i<=N-1 ; i++) {
			A.set(i-1, i, 1.0);
		}
		for(int i=0 ; i<N ; i++) {
			A.set(N-1, i, -poles.getPoly()[i]);
		}
		
		//A.print(5, 2);
		
		// Set Matrix b
		// All zeros except the last entry is 1
		b.set(N-1,0, zeroMatrix.get(N,0));
		
		
		// Markov parameters for initial conditions
		for(int n=0; n<=N; n++) {
			int index=n;
			for(int j=index; j>=0; j--) {
				F.set(n,n-j,poles.getPoly()[N-j]);
			}
		}

		Matrix G = F.inverse().times(zeroMatrix);
		
		Matrix X = new Matrix(N,401);
		for(int i=0; i<N; i++) {
			X.set(i,0,G.get(i,0));
		}
		// Verified Correct to this point
		//
		// Once initial conditions are set, begin loop
		Matrix K1 = new Matrix(N,1);
		Matrix K2 = new Matrix(N,1);
		Matrix K3 = new Matrix(N,1);
		Matrix K4 = new Matrix(N,1);
		
		for(int i=0; i<=399; i++) {
			K1=(A.times(X.getMatrix(0,N-1,i,i)).plus(b)).times(step);
			K2=(A.times(X.getMatrix(0,N-1,i,i).plus(K1.times(.5))).plus(b)).times(step);
			K3=(A.times(X.getMatrix(0,N-1,i,i).plus(K2.times(.5))).plus(b)).times(step);
			K4=(A.times(X.getMatrix(0,N-1,i,i).plus(K3)).plus(b)).times(step);
						
			//X(:,n+1) = X(:,n) + (K1 + 2*K2 + 2*K3 + K4)/6;
			for (int j=0; j<N; j++) {
				double temp =X.get(j,i)+(K1.get(j,0)+2.0*K2.get(j,0)+2*K3.get(j,0)+
								K4.get(j,0))/6.0;
				X.set(j,i+1,temp);
			}
		}

		// Pull out step response from first row of X matrix
		stepResponseData = X.getMatrix(0,0,0,400).getRowPackedCopy();
	}
	
	
	/**
	 * @return double[] representing the step response of the system
	 */
	public double[] getStepResponseData() {
		return stepResponseData;
	}
	
	/**
	 * Possible error when there are more zeros than poles.
	 * 
	 * @return true if there was an error calculating the step response
	 */
	public boolean getErrorMsg() {
		return errorMsg;
	}
	/**
	 * @return The gain of the current system
	 */
	public double getGain() {
		return gain;
	}

}
